@@ -367,7 +367,8 @@ std::string gguf_get_model_arch(const std::string & gguf_filename)
             {
                 fileformatmeta->model_architecture = GGUFArch::ARCH_FALCON;
             }
-            else if(modelarch=="mamba" || modelarch=="mamba2" || modelarch=="nemotron_h" || modelarch=="jamba") //lazy approach, put all RNN models
+            else if(modelarch=="mamba" || modelarch=="mamba2" || modelarch=="nemotron_h" || modelarch=="jamba" || modelarch=="granitehybrid" || modelarch=="lfm2"
+            || modelarch=="plamo2" || modelarch=="falcon-h1") //lazy approach, put all non rwkv RNN models
             {
                 fileformatmeta->model_architecture = GGUFArch::ARCH_MAMBALIKE;
             }
@@ -391,7 +392,7 @@ std::string gguf_get_model_arch(const std::string & gguf_filename)
             {
                 fileformatmeta->model_architecture = GGUFArch::ARCH_GEMMA3N;
             }
-            else if(modelarch=="rwkv6" || modelarch=="rwkv7")
+            else if(modelarch=="rwkv6" || modelarch=="rwkv7" || modelarch=="rwkv6qwen2" || modelarch=="arwkv7")
             {
                 fileformatmeta->model_architecture = GGUFArch::ARCH_RWKV;
             }
